#include "matrix.h"

#define DEFAULT_TOL 1e-10

#define _p(a) #a
int main(int argc, char** argv) {
  --argc; ++argv;
  if (argc < 2 || argc > 4) {
    printf("Usage:\n"
           "  cm.bin A.mx B.mx [tol [text_file]]\n"
           "Description:\n"
           "  - checks if A = B and prints out any errors.\n"
           "  - If tol is unspecified it defaults to " _p(DEFAULT_TOL) "\n"
           "  - If output text_file is unspecified, prints to stdout.\n");
    return 1;
  }
  const char* f1 = argv[0];
  const char* f2 = argv[1];
  double tol = argc >= 3 ? atof(argv[2]) : DEFAULT_TOL;
  if (tol <= 0.0) {
    printf("Invalid tolerance value: %f\n", tol);
    return 1;
  }

  FILE* fp = argc == 4 ? fopen(argv[3], "w") : stdout;
  if (!fp) {
    perror(argv[4]);
    return 1;
  }

  int m1, n1, m2, n2; 
  double **matrix1, **matrix2;

  printf("Reading first matrix ...\n");
  matrix1 = matrix_read(f1, &m1, &n1);
  if (!matrix1) {
    printf("Error reading %s!\n", f1);
    return 1;
  }
  
  printf("Reading second matrix ...\n");
  matrix2 = matrix_read(f2, &m2, &n2);
  if (!matrix2) {
    printf("Error reading %s!\n", f2);
    return 1;
  }

  int err;
  if (m1 != m2 || n1 != n2) {
    fprintf(fp, "Dimension mismatch: %d x %d vs %d x %d\n", m1, n1, m2, n2);
    err = 1;
  } else {
    printf("Comparing matrixes ...\n");
    err = matrix_compare(fp, matrix1, matrix2, m1, n1, tol);
  }
  if (fp != stdout) {
    fclose(fp);
    if (err) {
      printf("There were %d errors, see %s for details\n", err, argv[3]);
    } else {
      printf("No errors!");
      unlink(argv[3]);
    }
  }
  matrix_free(matrix1);
  matrix_free(matrix2);
  return err ? 1 : 0;
}
