#include "matrix.h"

int main(int argc, char** argv) {
  --argc; ++argv;
  if (argc < 1 || argc > 3) {
    printf("Usage:\n"
           "  gles.bin n [a [s]] \n"
           "Description:\n"
           "  - generates a n x (n+1) LES with non-diagonal values in [-a, a]\n"
           "  - the output may be fed to sles_seq.bin\n"
           "  - the last column is the vector part of the equation.\n" 
           "  - diagonal values may fall out of [-a,a].\n"
           "  - if unspecified, a defaults to 10\n"
           "  - if unspecified, the PRNG seed s defaults to time(0) %% 60\n");
    return 1;
  }

  int n = atoi(argv[0]);
  int a = argc >= 2 ? atoi(argv[1]) : 10;
  int seed = argc == 3 ? atoi(argv[2]) : time(0) % 60;
 
  if (n <= 0 || a <= 0) {
    printf("Invalid arguments\n");
    return 1;
  }
  printf("Generating matrix ...\n");
  double** matrix = matrix_gen(n, n, a, seed);
  if (! matrix) {
    printf("Error generating matrix\n");
    return 1;
  }
  // Now modify it such that it is diagonally-dominant.
  for (int i = 0; i < n; i++) {
    double s = 0;
    for (int j = 0; j < n; j++) {
      if (j != i) {
        s += fabs(matrix[i][j]);
      }
    }
    s += 1;
    matrix[i][i] = i % 2 == 0 ? s : -s; 
  }
  char file[PATH_MAX+1]; 
  sprintf(file, "A_%d_%d_%d.mx", n, a, seed);
  printf("Writing matrix to '%s' ...\n", file);
  matrix_write(file, matrix, n, n);
  matrix_free(matrix);
  // Generate vector
  double** v = matrix_gen(1, n, a, seed);
  sprintf(file, "b_%d_%d_%d.mx", n, a, seed);
  printf("Writing vector to '%s' ...\n", file);
  matrix_write(file, v, 1, n);
  matrix_free(v);
  return 0;
}
