
#ifndef __matrix_h__
#define __matrix_h__

// C library headers
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <math.h>
#include <limits.h>

#ifndef PATH_MAX
#define PATH_MAX 255
#endif

#ifdef _OPENMP
#include <omp.h>
#endif

// == Provided functions (see common.c) ==

// Allocation / deallocation
double** matrix_alloc(int m, int n);
void matrix_free(double** matrix);

// Reading & writing from/to files
double** matrix_read(const char* file, int* m, int* n);
void matrix_write(const char* file, double** matrix, int m, int n);

// Matrix printout (used by mprint.bin)
void matrix_print(FILE* fp, double** matrix, int m, int n);

// Pseudo-random matrix generation (used by mgen.bin)
double** matrix_gen(int m, int n, double a, int seed);

// Matrix comparison (used by mcmp.bin)
int matrix_compare(FILE* fp, double** a, double** b, int m, int n, double tol);

// wall time 
#ifdef _OPENMP
#define wall_time omp_get_wtime
#else
double wall_time(); 
#endif

// Functions to implement

void matrix_mul
(double** A, double** B, double** C, int m, int n, int p);

void matrix_solve_les(double** A, double* b, int n); 

void matrix_inverse(double** A, double** invA, int n);


#endif
