#include "matrix.h"

void mvp(double** M, double* v, double* r, int m, int n) {
  #pragma omp parallel for
  for (int i = 0; i < m; i++) {
    double s = 0;
    for (int j = 0; j < n; j++) {
      s += M[i][j] * v[j];
    }
    r[i] = s;
  }
}

int main(int argc, char** argv) {
  --argc; ++argv;
  if (argc != 3) {
    printf("Usage:\n"
           "  mvp.bin M.mx v.mx r.mx\n"
           "Description:\n"
           "  - computes the matrix-vector product of M and v onto r.mx\n");
    return 1;
  }
  const char* M_file = argv[0];
  const char* v_file = argv[1];
  const char* r_file = argv[2];

  int m, n, must_be_1, m1;
  printf("Reading M and v ...\n");
  double** M = matrix_read(M_file, &m, &n);
  double** v = matrix_read(v_file, &must_be_1, &m1);
  double** r = matrix_alloc(1, m);

  if (!M || !v || !r) {
    printf("Error initialising or reading data!\n");
    return 1;
  }
  if (must_be_1 != 1  && m1 != m) {
    printf("Invalid dimensions for M and v\n");
    return 1;
  }
  omp_set_num_threads(4);
  printf("Calculating matrix-vector product ...\n");
  mvp(M, *v, *r, m, n);
  printf("Writing result to %s ...\n", r_file);
  matrix_write(r_file, r, 1, m);
  matrix_free(M);
  matrix_free(v);
  matrix_free(r);
  return 0;
}
